package de.intevation.flys.client.client.ui;

import com.google.gwt.core.client.GWT;
import com.google.gwt.i18n.client.LocaleInfo;
import com.google.gwt.user.client.Window;

import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;

import de.intevation.flys.client.client.FLYS;
import de.intevation.flys.client.client.FLYSMessages;
import de.intevation.flys.client.shared.model.User;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class MainMenu extends HLayout {

    /** The interface that provides i18n messages. */
    private FLYSMessages messages = GWT.create(FLYSMessages.class);

    /** An instance to FLYS.*/
    protected FLYS flys;

    /** The user that is currently logged in. */
    protected User currentUser;

    /** The label that displays the current logged in user. */
    protected Label userText;

    /** The button to log the current user out.*/
    protected Label logout;

    /** The button to add new projects.*/
    protected Label newProject;

    /** The button to open the project list.*/
    protected Label projectList;

    /** The button to switch between the english and german version.*/
    protected Label language;

    /** The button to open an info panel.*/
    protected Label info;

    /**
     * The default constructor for creating a new MainMenu.
     */
    public MainMenu(FLYS flys) {
        this.flys     = flys;

        String guest = messages.user() + " " + messages.guest();

        userText    = new Label(guest);
        newProject  = new Label(messages.new_project());
        projectList = new Label(messages.manage_projects());
        logout      = new Label(messages.logout());
        language    = new Label(messages.switch_language());
        info        = new Label(messages.info());

        newProject.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                GWT.log("Clicked 'New Project' button.");
                createNewProject();
            }
        });

        projectList.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                GWT.log("Clicked 'Open ProjectList' button.");
                ProjectList list = getFlys().getProjectList();
                if (list.isVisible())
                    list.hide();
                else
                    list.show();
            }
        });

        logout.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                GWT.log("Clicked 'logout' button.");
                GWT.log("IMPLEMENT the 'logout' function.");
            }
        });

        language.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                LocaleInfo info      = LocaleInfo.getCurrentLocale();
                String currentLocale = info.getLocaleName();
                String newLocale     = currentLocale.equals("de") ? "en" : "de";

                // XXX  Maybe we should inform the user about fact, that we will
                // loose the work of the current session.
                switchLanguage(currentLocale, newLocale);
            }
        });

        info.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                GWT.log("Clicked 'info' button.");
                GWT.log("IMPLEMENT the 'open info panel' function.");
            }
        });

        init();
    }


    /**
     * This method is called by the constructor after creating the necessary
     * components. It initializes layout specific stuff like width, height,
     * colors and so on and appends the components.
     */
    protected void init() {
        setStyleName("bgBlueDark");
        setHeight("25px");
        setLayoutMargin(5);

        newProject.setStyleName("fontLightSmall");
        projectList.setStyleName("fontLightSmall");
        userText.setStyleName("fontLightSmall");
        newProject.setStyleName("fontLightSmall");
        logout.setStyleName("fontLightSmall");
        language.setStyleName("fontLightSmall");
        info.setStyleName("fontLightSmall");

        projectList.setWidth("140px");

        HLayout leftPanel = new HLayout();
        leftPanel.setWidth("80%");
        leftPanel.setMembersMargin(5);
        leftPanel.addMember(newProject);
        leftPanel.addMember(projectList);

        userText.setAlign(Alignment.RIGHT);
        logout.setAlign(Alignment.RIGHT);
        info.setAlign(Alignment.RIGHT);
        language.setAlign(Alignment.RIGHT);

        userText.setWidth(200);
        logout.setWidth(70);
        info.setWidth(40);
        language.setWidth(70);

        HLayout rightPanel = new HLayout();
        rightPanel.setAlign(Alignment.RIGHT);
        rightPanel.setMembersMargin(3);
        rightPanel.setLayoutRightMargin(5);
        rightPanel.addMember(userText);
        rightPanel.addMember(logout);
        rightPanel.addMember(language);
        rightPanel.addMember(info);

        addMember(leftPanel);
        addMember(rightPanel);
    }


    /**
     * Returns the FLYS instance stored in this class.
     *
     * @return the flys instance.
     */
    protected FLYS getFlys() {
        return flys;
    }


    /**
     * Set the current {@link User} and call {@link updateCurrentUser()}
     * afterwards.
     *
     * @param user the new user.
     */
    public void setCurrentUser(User currentUser) {
        this.currentUser = currentUser;

        updateCurrentUser();
    }


    /**
     * Update the text field that shows the current user. If no user is
     * currently logged in, the text will display {@link FLYSMessages.guest()}.
     */
    public void updateCurrentUser() {
        String name = currentUser != null
            ? currentUser.getName()
            : messages.guest();

        GWT.log("Update the current user: " + name);

        String username = messages.user() + " " + name;
        userText.setContents(username);
    }


    /**
     * Create a new project by calling FLYS.newProject().
     */
    protected void createNewProject() {
        flys.newProject();
    }


    /**
     * This method triggers the language switch between the <i>currentLocale</i>
     * and the <i>newLocale</i>. The switch is done by replacing a "locale="
     * parameter in the url of the application. We could use the GWT UrlBuilder
     * class to create a new URL, but - in my eyes - this class is a bit
     * inconsistens in its implementation.
     *
     * @param currentLocale The current locale string (e.g. "en").
     * @param newLocale The new locale string (e.g. "de").
     */
    protected void switchLanguage(String currentLocale, String newLocale) {
        String newLocation = Window.Location.getHref();

        if (newLocation.indexOf("&locale=") >= 0) {
            String replace = currentLocale.equals("de")
                ? "&locale=de"
                : "&locale=en";

            String replaceWith = newLocale.equals("de")
                ? "&locale=de"
                : "&locale=en";

            newLocation = newLocation.replace(replace, replaceWith);
        }
        else if (newLocation.indexOf("?locale=") >= 0) {
            String replace = currentLocale.equals("de")
                ? "?locale=de"
                : "?locale=en";

            String replaceWith = currentLocale.equals("de")
                ? "?locale=de"
                : "?locale=en";

            newLocation = newLocation.replace(replace, replaceWith);
        }
        else {
            newLocation += newLocation.indexOf("?") >= 0
                ? "&locale=" + newLocale
                : "?locale=" + newLocale;
        }

        Window.open(newLocation, "_self", "");
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
